/**
 * @file ext_int_driver.c
 * @brief External interrupt line driver
 *
 * @section License
 *
 * Copyright (C) 2010-2017 Oryx Embedded SARL. All rights reserved.
 *
 * This file is part of CycloneTCP Eval.
 *
 * This software is provided in source form for a short-term evaluation only. The
 * evaluation license expires 90 days after the date you first download the software.
 *
 * If you plan to use this software in a commercial product, you are required to
 * purchase a commercial license from Oryx Embedded SARL.
 *
 * After the 90-day evaluation period, you agree to either purchase a commercial
 * license or delete all copies of this software. If you wish to extend the
 * evaluation period, you must contact sales@oryx-embedded.com.
 *
 * This evaluation software is provided "as is" without warranty of any kind.
 * Technical support is available as an option during the evaluation period.
 *
 * @author Oryx Embedded SARL (www.oryx-embedded.com)
 * @version 1.8.0
 **/

//Dependencies
#include "stm32f10x.h"
#include "stm32f10x_gpio.h"
#include "stm32f10x_rcc.h"
#include "stm32_eval.h"
#include "stm3210c_eval_ioe.h"
#include "core/net.h"
#include "ext_int_driver.h"
#include "debug.h"


/**
 * @brief External interrupt line driver
 **/

const ExtIntDriver extIntDriver =
{
   extIntInit,
   extIntEnableIrq,
   extIntDisableIrq
};


/**
 * @brief EXTI configuration
 * @return Error code
 **/

error_t extIntInit(void)
{
   //Configure IO expander
   IOE_Config();
   //Configure PHY transceiver IRQ
   IOE_ITConfig(IOE_ITSRC_MII_IRQ);

   //Successful processing
   return NO_ERROR;
}


/**
 * @brief Enable external interrupts
 **/

void extIntEnableIrq(void)
{
   //Enable EXTI15_10 interrupt
   NVIC_EnableIRQ(EXTI15_10_IRQn);
}


/**
 * @brief Disable external interrupts
 **/

void extIntDisableIrq(void)
{
   //Disable EXTI15_10 interrupt
   NVIC_DisableIRQ(EXTI15_10_IRQn);
}


/**
 * @brief External interrupt handler
 **/

void EXTI15_10_IRQHandler(void)
{
   bool_t flag;
   NetInterface *interface;

   //Enter interrupt service routine
   osEnterIsr();

   //Point to the structure describing the network interface
   interface = &netInterface[0];
   //This flag will be set if a higher priority task must be woken
   flag = FALSE;

   //Check whether the specified EXTI line is asserted
   if(EXTI_GetITStatus(EXTI_LINE_IOE_ITLINE) != RESET)
   {
      //Clear interrupt flag
      EXTI_ClearITPendingBit(EXTI_LINE_IOE_ITLINE);

      //Check IO expander interrupt status
      if(IOE_GetGITStatus(IOE_2_ADDR, IOE_GIT_GPIO) != RESET)
      {
         //Clear interrupt flag
         IOE_ClearGITPending(IOE_2_ADDR, IOE_GIT_GPIO);

         //Checks the status of the MII_IRQ pin
         if(IOE_GetIOITStatus(IOE_2_ADDR, IOE_MII_IRQ_IT) != RESET)
         {
            //Clear interrupt flag
            IOE_ClearIOITPending(IOE_2_ADDR, IOE_MII_IRQ_IT);

            //Set event flag
            interface->phyEvent = TRUE;
            //Notify the TCP/IP stack of the event
            flag = osSetEventFromIsr(&netEvent);
         }
      }
   }

   //Leave interrupt service routine
   osExitIsr(flag);
}
